﻿import sys
import clr
import re
clr.AddReferenceToFile("DotPUniGenEngine.dll")
from tfm.DotPUniGen.Engine import *
from tfm.Common import *

sLocalPath = "C:\\Google Drive\\Community Wad Folders\\Community Wad Tools\\Working\\Spoilers\\Spoiled Cards\\"

def Info():
	return """Local Input X99
v1.2.1
by Xander9009
Extracts card information from local files in the specified directory. Can be used for spoiled cards or custom cards."""

def Purpose():
	return PluginPurpose.Input

def Get(sUserInput):
	crdCard = Card()
	sName = sUserInput
	sUserInput = sUserInput.replace("/", "_")
	
	sPage = ""
	try:
		with open(sLocalPath + sUserInput + ".txt", 'r') as myfile:
			sPage=myfile.read()
	except:
		# search failed: return empty card
		return crdCard
	RetrieveAllEnglishData(crdCard, sPage)
	return crdCard

def RetrieveAllEnglishData(crdCard, sPage):
	RetrieveTextData(crdCard, Language.English, sPage)
	# get id (if still missing) - Will never exist with spoilers.
	moMatch = re.search("MID: ([0-9]+)$", sPage, re.MULTILINE)
	if moMatch:
		crdCard.MultiverseID = moMatch.group(1)
	else:
		crdCard.MultiverseID = "0"
	# get expansion (if still missing)
	moMatch = re.search("Set: ([A-Z][A-Z][A-Z])$", sPage, re.MULTILINE)
	if moMatch:
		crdCard.Expansion = moMatch.group(1)
	# get rarity (if still missing)
	moMatch = re.search("Rarity: (.*)$", sPage, re.MULTILINE)
	if moMatch:
		crdCard.SetRarity(moMatch.group(1).replace(" ", ""))
	# get casting cost
	moMatch = re.search("Cost: (.*)$", sPage, re.MULTILINE)
	if moMatch:
		sCost = moMatch.group(1)
		# format phyrexian mana - This hasn't been converted to MTGSalvation's RSS feed.
		# sCost = re.sub(r'(?<=\{)(W|U|B|R|G)(?=P\})', r'\1/', sCost)
		# format "normal" mana
		sCost = re.sub(r'(?<!\{)((\d+)|W|U|B|R|G)(?!\})', r'{\1}', sCost)
		crdCard.CastingCost = sCost
	# get P/T
	moMatch = re.search("PT: (\S+)/(\S+?)$", sPage, re.MULTILINE)
	if moMatch:
		crdCard.Power = moMatch.group(1)
		crdCard.Toughness = moMatch.group(2)
	# get types, supertypes and sub-types
	moMatch = re.search("Type: (.*?)$", sPage, re.MULTILINE)
	if moMatch:
		sTypes = moMatch.group(1).replace(",", "")
		lstSubTypes = []
		iPos = sTypes.find('—')
		if iPos > -1:
			lstSubTypes = sTypes[iPos+1:].split()
			sTypes = sTypes[:iPos]
		else:
			iPos = sTypes.find('-')
			if iPos > -1:
				lstSubTypes = sTypes[iPos+1:].split()
				sTypes = sTypes[:iPos]
		lstTypes = sTypes.split()
		for sType in lstTypes:
			# one of the following will do nothing
			crdCard.AddType(sType)
			crdCard.AddSupertype(sType)
		for sSubType in lstSubTypes:
			crdCard.AddSubType(sSubType)
	# get artist
	moMatch = re.search("Artist: (.+?)$", sPage, re.MULTILINE)
	if moMatch:
		crdCard.Artist = moMatch.group(1)
	# get colors - Not converted for MTGSalvation spoilers.
	moMatch = re.search(r'\(Color Indicator: (.+?)\)$', sPage, re.MULTILINE)
	if moMatch:
		crdCard.ForcedColors = True
		lstColors = moMatch.group(1).split()
		for sColor in lstColors:
			crdCard.AddColor(sColor)
	else:
		for ccColor in MiscUtils.GetValues[CardColor]():
			if ccColor != CardColor.Colorless and crdCard.CastingCost.find(Constants.ColorToCode(ccColor)) > -1:
				crdCard.AddColor(ccColor)

def RetrieveTextData(crdCard, lLanguage, sPage):
	# get title
	moMatch = re.search("Name: (.*)$", sPage, re.MULTILINE)
	if moMatch:
		crdCard.SetTitle(lLanguage, StringUtils.GracefulQuotes(moMatch.group(1).strip()))
	# get flavor
	moMatch = re.search("Flavor: (.*)$", sPage, re.MULTILINE)
	if moMatch:
		sFlavor = moMatch.group(1)
		# add the return before a quoted sentence's author
		if lLanguage == Language.French:
			sFlavor = sFlavor.replace('»—', '»\n—')
		elif lLanguage == Language.Japanese:
			sFlavor = sFlavor.replace('」――', '」\n――')
		else:
			sFlavor = sFlavor.replace('"—', '"\n—')
		sFlavor = sFlavor.strip()
		crdCard.SetFlavor(lLanguage, StringUtils.GracefulQuotes(sFlavor))
	# get abilities
	moMatch = re.search("Rules: (.*)$", sPage, re.MULTILINE)
	if moMatch:
		# the endline marker below is actually being used by the page, don't change it
		sEndLine = '\\n'
		sAbilities = moMatch.group(1)
		sAbilities = re.sub(r'{w}', r'{W}', sAbilities)
		sAbilities = re.sub(r'{u}', r'{U}', sAbilities)
		sAbilities = re.sub(r'{b}', r'{B}', sAbilities)
		sAbilities = re.sub(r'{r}', r'{R}', sAbilities)
		sAbilities = re.sub(r'{g}', r'{G}', sAbilities)
		sAbilities = re.sub(r'{t}', r'{T}', sAbilities)
		# remove reminder text, while adding missing endline markers (if it wasn't missing, it will be removed later)
		sAbilities = re.sub(r'((?:\s*)\|\((?:.+?)(?:\.|。)\)\|?(?:\s*))', sEndLine, sAbilities)
		# add missing endline markers where a capital letter or a cost directly follows a non-capital letter
		sAbilities = re.sub(r'([a-z])([A-Z{])', r'\1' + sEndLine + r'\2', sAbilities)
		# add missing endline markers where a capital letter or a cost directly follows a full stop
		sAbilities = re.sub(r'(\.|。)([A-Z{])', r'\1' + sEndLine + r'\2', sAbilities)
		# remove redundant endline markers
		sAbilities = re.sub('(' + sEndLine + ')+', sEndLine, sAbilities)
		sAbilities = re.sub('(^' + sEndLine + '|' + sEndLine + '$)', "", sAbilities)
		
		sAbilities = re.sub(' +', ' ', sAbilities)
		
		# get loyalty ability
		moMatch = re.search("Loyalty: (.+)$", sPage, re.MULTILINE)
		if moMatch:
			sAbilities = "|(Loyalty " + moMatch.group(1) + ")|" + sEndLine + sAbilities
			
		# split at endline markers
		lstAbilities = sAbilities.split(sEndLine)
		
		for iIndex, sAbility in enumerate(lstAbilities):
			sAbility = sAbility.strip()
			if len(sAbility) > 0:
				crdCard.SetAbilityText(iIndex, lLanguage, StringUtils.GracefulQuotes(sAbility))

def Test():
	c = Get('devouring strossus')
	for i in range(0, c.GetAbilitiesCount()):
		print c.GetAbilityText(i, Language.English)
		
# 1.1
# 25-06-17 - Fixed loyalty ability being added when loyalty is blank.
# 25-06-17 - Allowed reminder text to be removed when trailing italicizing pipe is missing.
# 25-06-17 - Added check to reduce double spaces to single spaces.

# 1.2
# 25-06-17 - Fixed issue with typeline not properly registering "-" the same as "—".
# 25-06-17 - Fixed spaces on the end of names not being removed.

# 1.2.1
# 28-06-17 - Added MID support.